/*
 * Projekt: C-quest
 * Autor: David Martinek (c) 2004
 * email: martinek@fit.vutbr.cz
 * Licence: GPL v.2
 * Popis: Modul s funkcemi pro analzu a natn vstupnho souboru.
 *  
 * $Id: cqparse.c,v 1.11 2004/07/12 17:30:44 martinek Exp $
 * 
 * Copyright (C)2004 David Martinek
 *
 * Tento program je svobodn software; mete jej it a modifikovat podle
 * ustanoven GNU General Public License, vydvan Free Software
 * Foundation; a to bu verze 2 tto licence anebo (podle vaeho uven)
 * kterkoli pozdj verze.
 *
 * Tento program je roziovn v nadji, e bude uiten, avak BEZ
 * JAKKOLI ZRUKY; neposkytuj se ani odvozen zruky PRODEJNOSTI anebo
 * VHODNOSTI PRO URIT EL. Dal podrobnosti hledejte ve GNU General Public 
 * License.
 *
 * Kopii GNU General Public License jste ml obdret spolu s tmto
 * programem; pokud se tak nestalo, napite o ni Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "cqparse.h"
#include "cqtools.h"
#include <assert.h>
#include <string.h>

/** Znak oznaujc koment. V tto implementaci jde o znak mka (#) */
const char C_COMM = '#';

/** Znak oznaujc oddlova poloek v souboru. Jde o znak hvzdika (*). */
const char C_STAR = '*';

/** Chybov hlka udvajc, e ve je v podku. */
const char *NOERR =
"Soubor je v podku.\n";

/** Varovn, e v souboru jsou pli dlouh dky. */
const char *WARN_LONGLINE =
"V souboru se vyskytuj dky del ne 78 znak!\n";

/** Chyb nadpis testu. */
const char *ERR_TITLE = 
"Chyb nadpis souboru!\n";

/** Chyba nkde v otzce. */
const char *ERR_QUEST = 
"Chyb otzka nebo zabr pli mnoho dk!\n";

/** Chyba nkde ve variantch odpovd. */
const char *ERR_VAR = 
"Chyb varianty odpovd nebo je jich pli mnoho!\n";

/** Chybn poad variant odpovd. */
const char *ERR_VARORDER = 
"Varianty odpovd nejsou ve sprvnm poad!\n";

/** Chyba na dku se sprvnou odpovd a asem. */
const char *ERR_ANSWTIME = 
"Chyba pi natn kdu odpovdi nebo asov nronosti otzky!\n";

/** Chyba nkde v textu s vysvtlenm sprvn odpovdi. */
const char *ERR_EXPL = 
"Chyb vysvtlen sprvn odpovdi nebo zabr pli mnoho dk!\n";

/** Chyb oddlova (hvzdika) v mst, kde byl oekvn. */
const char *ERR_SEP = 
"Chyb oddlova (hvzdika) v mst, kde byl oekvn.\n";

/** Neoekvan konec souboru. */
const char *ERR_EOF =
"Neoekvan konec souboru!\n";

/** Chyba na dku ... */
const char *ERR_ONLINE = 
"Chyba vznikla na dku %d\n";

/**
 * Pesko dek souboru. Pokud naraz na konec souboru, vrac EOF. Pokud je
 * ve v podku, poslednm petenm znakem je '\n' a pi dalm ten znaku
 * se tedy nate prvn znak dalho dku.
 */
static inline int skipLine(FILE *f)
{
  int c;
  while ((c = fgetc(f)) != '\n')
  {
    if (c == EOF) return c;
  };
  return c;
}

/**
 * Nate ze souboru dal nezakomentovan dek. Pokud naraz na konec souboru,
 * vrac EOF. Parametr line mus ukazovat na sprvn alokovanou pam. Po 
 * peten dku bude ukazatel v souboru ped prvnm znakem dalho dku. 
 * Na konci natenho dku bude vdy znak "\n".
 *
 * @param f Ukazatel na popisova souboru.
 * @param nLines Poet petench dk. Funkce zv toto slo o poet 
 *   dk, kter nate.
 * @param line Buffer, do nj se natou data. Je nutn, aby v nm byl alokovn
 *   dostatek pamti. Peten dek bude vdy konit sekvenc "\n\0". Obsah
 *   tohoto bufferu je hlavnm vsledkem voln funkce.
 * @param max Rozmr bufferu line. Mus to bt minimln 3, aby bylo mon
 *   i za dek, kter je del doplnit "\n\0".
 * @return Vrac EOF, pokud bhem ten naraz na konec souboru nebo prvn
 *   znak petenho dku.
 */
int readLine(FILE *f, int *nLines, char line[], int max)
{
  assert(f != NULL && line != NULL && max > 3);
  
  int c;
  
  do  // peskakuju komente a vracm prvn dek, kter nen zakomentovan
  {
    *nLines += 1;
    if (fgets(line, max, f) == NULL) return EOF;
    
    int lineLen = strlen(line);
    if (line[lineLen-1] != '\n')
    { // pokud je dek moc dlouh, '\n' se nenate, je teba jej doplnit
      // a peskoit zbytek dku
      line[lineLen-1] = '\n'; 
      if (skipLine(f) == EOF) return EOF;
    }
  } while((c = line[0]) == C_COMM);

  return c; 
}

/**
 * Nate ze souboru maximln maxLines dk. Za natenmi dky oekv dek
 * zanajc hvzdikou, kter se nate, ale neulo. Do parametru lines se 
 * ulo etzec slepen z natench dk, je v nm budou oddleny znakem
 * konce dku "\n". Tato funkce alokuje pam!
 *
 * Vrac EOF, pokud bhem ten naraz na konec souboru nebo prvn znak
 * poslednho natenho dku (slou pro rozlien jestli se naetl dek
 * s hvzdikou nebo ne). Pokud vrt znak *, je ve v podku, v ostatnch
 * ppadech je nco patn.
 *
 * @param f Ukazatel na popisova souboru.
 * @param nLines Poet petench dk. Funkce zv toto slo o poet 
 *   dk, kter nate.
 * @param firstLine prvn naten dek, bude zkoprovn na prvn msto
 *   vslednho etzce.
 * @param lines Ukazatel na textov etzec. Tato funkce sama alokuje pam,
 *   take parametr by pi voln neml ukazovat na dnou alokovanou pam.
 * @param maxLines Maximln poet dk, kter je mon nast.
 * @return Vrac prvn znak poslednho natenho dku nebo EOF, pokud pi
 *   ten naraz na konec souboru.
 */
int readLines(FILE *f, int *nLines, char *firstLine, char **lines, int maxLines)
{
  assert(f != NULL && firstLine != NULL && maxLines > 0);
  
  int c;
  char line[LLEN] = {0};
  char lbuf[LLEN*maxLines];
  lbuf[0] = '\0';
  strcat(lbuf, firstLine);

  for (int i = 0; i < maxLines; i++)
  {
    c = readLine(f, nLines, line, LLEN);
    if (c == C_STAR) break;
    if (c == EOF) return c;
    strcat(lbuf, line);  // pid natenou dku na konec bufferu
  }

  *lines = myStrdup(lbuf); // alokuje pam a zkopruje do n obsah bufferu
  
  if (c != C_STAR) // asi se naetlo vech maxLines dk, mla by nsledovat *
  {
    c = readLine(f, nLines, line, LLEN);
  }
  
  return c;
}

/**
 * Vrac true, pokud vechny varianty odpovd maj jako prvn znak mal 
 * psmeno od a do z ve sprvnm poad, jinak vrac false.
 */
bool testAnswers(char *answers)
{
  assert(answers != NULL);
  
  char tmp[strlen(answers)+1];
  strcpy(tmp, answers);
  char *line = strtok(tmp, "\n"); // provd jednoduchou analzu textu
  char counter = 'a';

  while (line != NULL)
  {
    if (line[0] != counter++) return false;
    line = strtok(NULL, "\n");
  }
  return true;
}

/**
 * Nate data ze souboru. Vrac true, pokud ve probhlo bez problm nebo
 * vrac false, pokud je soubor ve patnm formtu.
 *
 * @return Vrac strukturu chybovou zprvou a s slem dku, kde tato chyba
 *   vznikla. Pokud je chybov zprva NOERR, k dn chyb nedolo a ve
 *   sloce .line je poet dk souboru (me se liit o jedniku od 
 *   skutenosti - zle na tom, jestli konec dku le na konci nebo na
 *   zatku dku).
 */
TErrorCode readTest(FILE *f, TTestData *data)
{
  assert(f != NULL && data != NULL);

  TErrorCode errStatus =
  {
    .errMsg = NOERR,
    .line = 0
  };

  enum states {TITLE, START_ITEM, QUEST, VARIANT, ANSWER, EXPLANATION};
  int state = TITLE;
  int c;
  char line[LLEN] = {0};
  TItem *item = NULL;
  
  while ((c = readLine(f, &errStatus.line, line, LLEN)) != EOF)
  {
    switch (state)
    { // tady se nate nadpis testu
      case TITLE: 
        if (c == C_STAR) // * tu bt nem -> chyb nadpis
        {
          errStatus.errMsg = ERR_TITLE;
          return errStatus;
        }
        
        data->title = myStrdup(line);
        state = START_ITEM;
        break;
      // zatek poloky, tady se alokuje pam pro poloku seznamu 
      case START_ITEM:
        if (c != C_STAR)
        {
          bool endFile = false;
          if (c == '\n') // tohle se me vyskytnout na konci souboru
          {
            endFile = fgetc(f) == EOF;
            if (endFile) break;
          }
          
          if (!endFile)
          { // pokud tu nen *, je to chyba
            errStatus.errMsg = ERR_SEP;
            return errStatus; 
          }
        }
        
        // vkldm ihned, abych to v ppad chyby mohl uvolnit
        item = createItem();
        addItem(data, item); 
        
        state = QUEST;
        break;
      case QUEST:
        c = readLines(f, &errStatus.line, line, &item->questLines, RLEN - 1);
        if (c == EOF) break;
        if (c != C_STAR)
        {
          errStatus.errMsg = ERR_QUEST;
          return errStatus;
        }
        
        state = VARIANT;
        break;
      case VARIANT:
        c = readLines(f, &errStatus.line, line, &item->answLines, 'z'-'a'+1);
        if (c == EOF) break;
        if (c != C_STAR) 
        {
          errStatus.errMsg = ERR_VAR;
          return errStatus;
        }

        if (!testAnswers(item->answLines))
        {
          errStatus.errMsg = ERR_VARORDER;
          return errStatus;
        }
        
        state = ANSWER;
        break;
      case ANSWER:
        {
          char answer;
          unsigned short time;
          c = sscanf(line, "%c %hud", &answer, &time);
          if (c != 2) // nenaetly se oba parametry
          {
            errStatus.errMsg = ERR_ANSWTIME;
            return errStatus;
          }
          item->rightAnswer = answer;
          item->time = time;
          c = readLine(f, &errStatus.line, line, LLEN);
          if (c == EOF) break;
          if (c != C_STAR)
          {
            errStatus.errMsg = ERR_SEP;
            return errStatus;
          }
          state = EXPLANATION;
        }
        break;
      case EXPLANATION:
        c = readLines(f, &errStatus.line, line, &item->explLines, RLEN - 1);
        if (c == EOF) break;
        if (c != C_STAR) 
        {
          errStatus.errMsg = ERR_EXPL;
          return errStatus;
        }
        
        state = START_ITEM;
        break;
    }
    
    if (c == EOF) break;
  }
  
  if (data->total == 0 || state != START_ITEM)
  { // buto je soubor przdn nebo kon uprosted njak poloky
    errStatus.errMsg = ERR_EOF;
  }
  
  return errStatus;
}

