/*
 * Projekt: C-quest
 * Autor: David Martinek (c) 2004
 * email: martinek@fit.vutbr.cz
 * Licence: GPL v.2
 * Popis: C-quest je program pro vuku a testovn znalost o jazyce C (ale
 * je mon jej pout pro vytven test z jakkoli oblasti).
 *
 * $Id: cquest.c,v 1.15 2004/07/06 21:49:41 martinek Exp $
 *
 * Copyright (C)2004 David Martinek
 *
 * Tento program je svobodn software; mete jej it a modifikovat podle
 * ustanoven GNU General Public License, vydvan Free Software
 * Foundation; a to bu verze 2 tto licence anebo (podle vaeho uven)
 * kterkoli pozdj verze.
 *
 * Tento program je roziovn v nadji, e bude uiten, avak BEZ
 * JAKKOLI ZRUKY; neposkytuj se ani odvozen zruky PRODEJNOSTI anebo
 * VHODNOSTI PRO URIT EL. Dal podrobnosti hledejte ve GNU General Public 
 * License.
 *
 * Kopii GNU General Public License jste mli obdret spolu s tmto
 * programem; pokud se tak nestalo, napite o ni Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <assert.h>

#include "cqlist.h"
#include "cqhelp.h"
#include "cqparse.h"
#include "cqtools.h"
#include "cqlearntest.h"

/** Mdy programu. */
enum modes 
{
  TEST_MODE, /**< Testovac md. */
  LEARN_MODE /**< Md uen. */
};

/** Struktura s parametry programu. */
typedef struct params
{
  enum modes mode; /**< Md programu. Me nabvat hodnot TEST_MODE a LEARN_MODE */
  char *fileName;  /**< Jmno souboru s testem. */
} TParams;

/**
 * Zpracuje parametry programu. Pokud jsou parametry ve patnm formtu, 
 * zobraz npovdu a ukon program. Pokud jsou parametry v podku, vrt
 * jako vsledek strukturu s hodnotami .mode - md programu a .fileName -
 * jmno souboru s testovmi otzkami.
 * 
 * @param argc Poet parametr pkazov dky
 * @param argv Parametry pkazov dky (na indexu 0 je jmno programu)
 * @return Vrac strukturu obsahujc jmno souboru a zjitn md
 */
TParams doParams(int argc, char *argv[])
{
  assert(argv != NULL);
  if (argc <= 1 || argc > 3) // patn poet parametr -> zbyten pokraovat
  {
    printError(BAD_PARAM);
    printHelp(VERSION);
    printHelp(HEAD);
    printHelp(PARAMS);
    exit(1);
  }

  TParams params = 
  {
    .mode = TEST_MODE,
    .fileName = NULL
  };

  // pro detekci chybovch stav
  bool param = false; 
  bool file = false;

  // od 1, protoe prvn parametr je jmno tohoto programu
  for (int i = 1; i < argc; i++)
  {
    // testovac md
    if (!param && strcmp( "-t", argv[i]) == 0) 
    {
      params.mode = TEST_MODE;
      param = true;
    }
    // md uen
    else if (!param && strcmp("-l", argv[i]) == 0)
    {
      params.mode = LEARN_MODE;
      param = true;
    }
    // tisk verze programu
    else if (strcmp("-v", argv[i]) == 0)
    {
      printHelp(VERSION);
      exit(0);
    }
    // tisk npovdy
    else if (strcmp("-h", argv[i]) == 0)
    {
      printHelp(VERSION);
      printHelp(HEAD);
      printHelp(PARAMS);
      exit(0);
    }
    // neznm parametr nebo druh parametr, kter se tv jako soubor
    // nebo druh parametr zanajc -
    else if ((param && argv[i][0] == '-') || file)
    {
      printError(BAD_PARAM);
      printHelp(PARAMS);
      exit(1);
    }
    // take to mus bt soubor
    else
    {
      params.fileName = argv[i];
      file = true;
    }
  }
  
  if (!file) // nenaetl soubor -> nkde je chyba
  {
    printError(BAD_PARAM);
    printHelp(PARAMS);
    exit(1);
  }

  return params;
}

/**
 * Zatek programu cquest.
 * 
 * @param argc Poet parametr pkazov dky.
 * @param argv Parametry pkazov dky.
 * @return Nvratov kd programu.
 */
int main(int argc, char *argv[])
{
  TParams params = doParams(argc, argv);

  FILE *f;
 
  if ((f = fopen(params.fileName, "r")) == NULL)
  { // chyba pi otevrn souboru
    printError(BAD_FILE);
    return EXIT_FAILURE;
  }

  TTestData *data = createTestData();
  TErrorCode errStatus = readTest(f, data);
 
  fclose(f); // zavu soubor, jakmile jej nepotebuji
  if (errStatus.errMsg != NOERR && errStatus.errMsg != WARN_LONGLINE)
  { // chyba pi ten souboru - je poteba po sob uklidit!
    printError(errStatus.errMsg);
    printError(ERR_ONLINE, errStatus.line);
    freeTestData(data);
    free(data);
    return EXIT_FAILURE;
  }

  if (errStatus.errMsg == WARN_LONGLINE)
  {
    printError(errStatus.errMsg);
  }

  countTotalTime(data);

  if (params.mode == TEST_MODE)
  { // testovac md
    runTest(data);
  }
  else
  { // vukov md
    runLearning(data);
  }

  // klid pamti
  freeTestData(data);
  free(data);
  return EXIT_SUCCESS;
}



